--- elFinder.class.php	2010-03-22 05:03:25.000000000 +0300
+++ elFinder-json.class.php	2010-10-17 14:39:31.986023001 +0400
@@ -278,10 +278,10 @@
 	 **/
 	public function run() {
 		if (empty($this->_options['root']) || !is_dir($this->_options['root'])) {
-			exit(json_encode(array('error' => 'Invalid backend configuration')));
+			exit(elFinder::encode(array('error' => 'Invalid backend configuration')));
 		}
 		if (!$this->_isAllowed($this->_options['root'], 'read')) {
-			exit(json_encode(array('error' => 'Access denied')));
+			exit(elFinder::encode(array('error' => 'Access denied')));
 		}
 		
 		$cmd = '';
@@ -293,11 +293,11 @@
 		if (!$cmd && $_SERVER["REQUEST_METHOD"] == 'POST') {
 			header("Content-Type: text/html");
 			$this->_result['error'] = 'Data exceeds the maximum allowed size';
-			exit(json_encode($this->_result));
+			exit(elFinder::encode($this->_result));
 		}
 		
 		if ($cmd && (empty($this->_commands[$cmd]) || !method_exists($this, $this->_commands[$cmd]))) {
-			exit(json_encode(array('error' => 'Unknown command')));
+			exit(elFinder::encode(array('error' => 'Unknown command')));
 		}
 		
 		if (isset($_GET['init'])) {
@@ -357,7 +357,7 @@
 		}
 		header("Content-Type: ".($cmd == 'upload' ? 'text/html' : 'application/json'));
 		header("Connection: close");
-		echo json_encode($this->_result);
+		echo elFinder::encode($this->_result);
 		
 		if (!empty($this->_options['logger']) && in_array($cmd, $this->_loggedCommands)) {
 			$this->_options['logger']->log($cmd, empty($this->_result['error']), $this->_logContext, !empty($this->_result['error']) ? $this->_result['error'] : '', !empty($this->_result['errorData']) ? $this->_result['errorData'] : array()); 
@@ -1853,9 +1854,97 @@
 	{
 		$time = explode(" ", microtime());
 		return (double)$time[1] + (double)$time[0];
-	}	
+	}
+    
+    private function encode($data) {
+		if (function_exists('json_encode')) {
+			return json_encode($data);
+		} else {
+			switch (gettype($data)) {
+				case 'boolean':
+					return $data ? 'true' : 'false';
+    			case 'integer':
+    			case 'double':
+      				return $data;
+    			case 'resource':
+    			case 'string':
+      				return '"' . str_replace(array("\r", "\n", "<", ">", "&"), array('\r', '\n', '\x3c', '\x3e', '\x26'), addslashes($data)) . '"';
+				case 'array':
+					if (empty($data) || array_keys($data) === range(0, sizeof($data) - 1)) {
+						$output = array();
+						
+						foreach ($data as $value) {
+							$output[] = elFinder::encode($value);
+						}
+						
+						return '[ ' . implode(', ', $output) . ' ]';
+					}
+    			case 'object':
+      				$output = array();
+      				
+					foreach ($data as $key => $value) {
+        				$output[] = elFinder::encode(strval($key)) . ': ' . elFinder::encode($value);
+					}
+					
+					return '{ ' . implode(', ', $output) . ' }';
+				default:
+					return 'null';
+			}
+		}
+	}
 	
-}
+	private function decode($json, $assoc = FALSE) {
+		if (function_exists('json_decode')) {
+			return json_decode($json);
+		} else {
+			$match = '/".*?(?<!\\\\)"/';
+
+			$string = preg_replace($match, '', $json);
+			$string = preg_replace('/[,:{}\[\]0-9.\-+Eaeflnr-u \n\r\t]/', '', $string);
 
+			if ($string != '') {
+				return NULL;
+			}
+
+			$s2m = array();
+			$m2s = array();
+
+			preg_match_all($match, $json, $m);
+			
+			foreach ($m[0] as $s) {
+				$hash = '"' . md5($s) . '"';
+				$s2m[$s] = $hash;
+				$m2s[$hash] = str_replace('$', '\$', $s);
+			}
+
+			$json = strtr($json, $s2m);
+
+			$a = ($assoc) ? '' : '(object) ';
+			
+			$data = array(
+				':' => '=>', 
+				'[' => 'array(', 
+				'{' => "{$a}array(", 
+				']' => ')', 
+				'}' => ')'
+			);
+			
+			$json = strtr($json, $data);
+  
+  			$json = preg_replace('~([\s\(,>])(-?)0~', '$1$2', $json);
+  
+ 			$json = strtr($json, $m2s);
+
+  			$function = @create_function('', "return {$json};");
+  			$return = ($function) ? $function() : NULL;
+
+  			unset($s2m); 
+			unset($m2s); 
+			unset($function);
+
+  			return $return;
+		}
+	}	
+}
 
 ?>
